#region Using declarations

// ReSharper disable RedundantUsingDirective
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Forms.VisualStyles;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using System.Globalization;
using System.Diagnostics;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
using NinjaTrader.NinjaScript.MarketAnalyzerColumns;
using Brush = System.Windows.Media.Brush;
using AB_PowerX = NinjaTrader.NinjaScript.AddOns.AB_PowerX;

// ReSharper restore RedundantUsingDirective

#endregion

#region LICENSE

/*
"AB-PowerX" and associated add-on file is a NinjaTrader Indicator designed to provide a paint-bar indication of trend direction and also provide trade signal detection and automatic risk/reward display. This indicator is based on Markus Heitkoetter's PowerX System.

Copyright 2021 Asif Bacchus <asif@asifbacchus.dev>

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This software was designed and authored by Asif Bacchus and has also been publicly released under the terms as stated above ('MIT License'). Please contact the author directly at asif@asifbacchus.dev for further information or to inquire about proprietary licensing. Please refer to the private git repo at 'https://git.asifbacchus.dev/asif/NinjaTrader-PowerX.git' to submit enhancements, suggestions, bug reports, etc.
*/

#endregion


//This namespace holds Indicators in this folder and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators.ABPowerX
{
    // ReSharper disable once InconsistentNaming
    // ReSharper disable RedundantNameQualifier
    [Gui.CategoryOrder(ParamGroupDisplay, 1)]
    [Gui.CategoryOrder(ParamGroupIndicators, 2)]
    [Gui.CategoryOrder(ParamGroupMacd, 5)]
    [Gui.CategoryOrder(ParamGroupStochastic, 6)]
    [Gui.CategoryOrder(ParamGroupRsi, 7)]
    [Gui.CategoryOrder(ParamGroupTradeHelpers, 10)]
    [Gui.CategoryOrder(ParamGroupAccount, 11)]
    [Gui.CategoryOrder(ParamGroupTriggerLine, 12)]
    [Gui.CategoryOrder(ParamGroupTradePlanner, 13)]
    [Gui.CategoryOrder("Data Series", 20)]
    [Gui.CategoryOrder("Setup", 21)]
    [Gui.CategoryOrder("Visual", 22)]
    [TypeConverter("NinjaTrader.NinjaScript.Indicators.ABPowerX.ShowHideParamsTypeConverter")]
    // ReSharper restore RedundantNameQualifier
    public class ABPowerX : Indicator
    {
        #region Variables

        private const string MyIndicatorName = "ABPowerX";

        private const string MyIndicatorDescription =
            @"Based on Markus Heitkoetter's PowerX System. Paint-bar indicator along with trade signal detection and automatic risk/reward calculation and display.";

        private const string MyIndicatorVersion = "0.7";

        private const string IncompatibleChartType = @"AB-PowerX indicator cannot render on this chart type";

        private Account account;
        private double? accountBalance;

        private int minBarsNeeded;
        private readonly List<AB_PowerX.PowerXBar> powerXBars = new List<AB_PowerX.PowerXBar>();
        private readonly List<AB_PowerX.PowerXBar> triggeredBars = new List<AB_PowerX.PowerXBar>();

        // property-related
        private const string ParamGroupDisplay = "Display Options";
        private const string ParamGroupIndicators = "Indicator Settings";
        private const string ParamGroupMacd = "MACD Parameters";
        private const string ParamGroupStochastic = "Stochastics Parameters";
        private const string ParamGroupRsi = "RSI Parameters";
        private const string ParamGroupTradeHelpers = "Trade Helpers";
        private const string ParamGroupTriggerLine = "Trigger Lines";
        private const string ParamGroupTradePlanner = "Trade Planner";
        private const string ParamGroupAccount = "Account and Risk";

        #endregion

        #region Parameters

        // ReSharper disable MemberCanBePrivate.Global

        // display options
        [XmlIgnore]
        [Display(Name = "Uptrend colour", Description = "Colour to paint up-trending bars.",
            GroupName = ParamGroupDisplay, Order = 1)]
        public Brush ColourUp { get; set; }

        [Browsable(false)]
        public string ColourUpSerialize
        {
            get { return Serialize.BrushToString(ColourUp); }
            set { ColourUp = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(Name = "Downtrend colour", Description = "Colour to paint down-trending bars.",
            GroupName = ParamGroupDisplay, Order = 2)]
        public Brush ColourDown { get; set; }

        [Browsable(false)]
        public string ColourDownSerialize
        {
            get { return Serialize.BrushToString(ColourDown); }
            set { ColourDown = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(Name = "Neutral colour", Description = "Colour to paint non-trending bars.",
            GroupName = ParamGroupDisplay, Order = 3)]
        public Brush ColourNeutral { get; set; }

        [Browsable(false)]
        public string ColourNeutralSerialize
        {
            get { return Serialize.BrushToString(ColourNeutral); }
            set { ColourNeutral = Serialize.StringToBrush(value); }
        }

        // ReSharper disable RedundantNameQualifier
        [NinjaScriptProperty]
        [TypeConverter(typeof(AB_PowerX.EnumDescriptionTypeConverter))]
        [PropertyEditor("NinjaTrader.Gui.Tools.StringStandardValuesEditorKey")]
        [Display(Name = "Candle fill", Description = "Directionality of candle to fill.", GroupName = ParamGroupDisplay,
            Order = 5)]
        public AB_PowerX.CandleFillStyle CandleFill { get; set; }

        // ReSharper restore RedundantNameQualifier

        // indicator options
        [NinjaScriptProperty]
        [Display(Name = "Customize indicator parameters", Description = "Allow editing indicator parameter values.",
            GroupName = ParamGroupIndicators, Order = 0)]
        [RefreshProperties(RefreshProperties.All)]
        public bool CustomizeIndicatorValues { get; set; }

        // ReSharper disable InconsistentNaming
        [NinjaScriptProperty]
        [Display(Name = "MACD fast average", Description = "MACD Fast moving average period.",
            GroupName = ParamGroupMacd, Order = 1)]
        public int MACDFast { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "MACD slow average", Description = "MACD slow moving average period.",
            GroupName = ParamGroupMacd, Order = 2)]
        public int MACDSlow { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "MACD signal average", Description = "MACD signal-line moving average period.",
            GroupName = ParamGroupMacd, Order = 3)]
        public int MACDSignal { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Stochastic %K", Description = "Period for Stochastic %K line.",
            GroupName = ParamGroupStochastic, Order = 4)]
        public int StochasticK { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Stochastic %D", Description = "Period for Stochastic %D line.",
            GroupName = ParamGroupStochastic, Order = 5)]
        public int StochasticD { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Stochastic smoothing", Description = "Period for moving average of Stochastic.",
            GroupName = ParamGroupStochastic, Order = 6)]
        public int StochasticSmooth { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "RSI period", Description = "Period for Relative Strength Index.",
            GroupName = ParamGroupRsi, Order = 7)]
        public int RSIPeriod { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "RSI smoothing", Description = "Period for moving average of Relative Strength Index.",
            GroupName = ParamGroupRsi, Order = 8)]
        public int RSISmooth { get; set; }
        // ReSharper restore InconsistentNaming

        // trade helper
        [NinjaScriptProperty]
        [Display(Name = "Trade helper", Description = "Display trade helper on chart.",
            GroupName = ParamGroupTradeHelpers,
            Order = 0)]
        [RefreshProperties(RefreshProperties.All)]
        public bool DisplayTradeHelper { get; set; }

        // ReSharper disable once RedundantNameQualifier
        [NinjaScriptProperty]
        [TypeConverter(typeof(NinjaTrader.NinjaScript.AccountNameConverter))]
        [Display(Name = "Account", Description = "Account from which to receive balance data.",
            GroupName = ParamGroupAccount, Order = 1)]
        public string AccountName { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Risk %",
            Description = "Percent of account to risk on each trade. The provided number will be divided by 100.",
            GroupName = ParamGroupAccount, Order = 2)]
        public float AccountPercentRisk { get; set; }

        [NinjaScriptProperty]
        [TypeConverter(typeof(AB_PowerX.EnumDescriptionTypeConverter))]
        [PropertyEditor("NinjaTrader.Gui.Tools.StringStandardValuesEditorKey")]
        [Display(Name = "Trigger lines", Description = "Options for which trigger lines are drawn.",
            GroupName = ParamGroupTriggerLine, Order = 1)]
        public AB_PowerX.TriggerDisplayOptions TriggerDisplayOptions { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Entry trigger distance",
            Description = "Number of ticks away from previous bar to set trigger point.",
            GroupName = ParamGroupTriggerLine,
            Order = 5)]
        public int TradeTriggerDistance { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Validity period",
            Description = "Number of bars a trigger should remain valid before being discarded.",
            GroupName = ParamGroupTriggerLine, Order = 6)]
        public int TradeTriggerValidityPeriod { get; set; }

        [XmlIgnore]
        [Display(Name = "Bullish trigger line", Description = "Style to draw bullish trigger lines.",
            GroupName = ParamGroupTriggerLine, Order = 10)]
        public Brush TradeBullishTriggerLine { get; set; }

        [Browsable(false)]
        public string TradeBullishTriggerLineSerialize
        {
            get { return Serialize.BrushToString(TradeBullishTriggerLine); }
            set { TradeBullishTriggerLine = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(Name = "Bearish trigger line", Description = "Style to draw bearish trigger lines.",
            GroupName = ParamGroupTriggerLine, Order = 11)]
        public Brush TradeBearishTriggerLine { get; set; }

        [Browsable(false)]
        public string TradeBearishTriggerLineSerialize
        {
            get { return Serialize.BrushToString(TradeBearishTriggerLine); }
            set { TradeBearishTriggerLine = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "Trigger line width", Description = "Width (in pixels) to draw trade trigger lines.",
            GroupName = ParamGroupTriggerLine, Order = 12)]
        public int TradeTriggerLineWidth { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Dashed trigger lines", Description = "Draw trigger lines as dashed instead of solid.",
            GroupName = ParamGroupTriggerLine, Order = 15)]
        public bool TradeTriggerLineDashed { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Mark trade entries", Description = "Display an arrow above/below entry bars.",
            GroupName = ParamGroupTriggerLine, Order = 20)]
        public bool TradeShowMarkers { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Show trade planner",
            Description = "Show or hide trade planner levels (stop-loss, profit target).",
            GroupName = ParamGroupTradePlanner, Order = 0)]
        [RefreshProperties(RefreshProperties.All)]
        public bool DisplayTradePlanner { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "ADR period",
            Description = "Period for the Average Daily (Bar) Range indicator used for setting the stop and target.",
            GroupName = ParamGroupTradePlanner, Order = 5)]
        public int TradePlannerAdrPeriod { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "ADR stop multiple", Description = "Multiple of ADR to use when calculating the stop level.",
            GroupName = ParamGroupTradePlanner, Order = 6)]
        public double TradePlannerStopMultiple { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "ADR target multiple",
            Description = "Multiple of ADR to use when calculating the target level.",
            GroupName = ParamGroupTradePlanner, Order = 7)]
        public double TradePlannerTargetMultiple { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Line length (bars)", Description = "Number of bars to the right to extend planning lines.",
            GroupName = ParamGroupTradePlanner, Order = 8)]
        public int TradePlannerLineLength { get; set; }

        [XmlIgnore]
        [Display(Name = "Stop level", Description = "Colour for stop level line.", GroupName = ParamGroupTradePlanner,
            Order = 10)]
        public Brush TradePlannerStopLine { get; set; }

        [Browsable(false)]
        public string TradePlannerStopLineSerialize
        {
            get { return Serialize.BrushToString(TradePlannerStopLine); }
            set { TradePlannerStopLine = Serialize.StringToBrush(value); }
        }

        [XmlIgnore]
        [Display(Name = "Target level", Description = "Colour for target level line.",
            GroupName = ParamGroupTradePlanner, Order = 11)]
        public Brush TradePlannerTargetLine { get; set; }

        [Browsable(false)]
        public string TradePlannerTargetLineSerialize
        {
            get { return Serialize.BrushToString(TradePlannerTargetLine); }
            set { TradePlannerTargetLine = Serialize.StringToBrush(value); }
        }

        [NinjaScriptProperty]
        [Display(Name = "Line width", Description = "Thickness of line/dots drawn for stop and target lines.",
            GroupName = ParamGroupTradePlanner, Order = 12)]
        public int TradePlannerLineThickness { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Dotted planning lines", Description = "Draw stop and target lines as dots instead of solid.",
            GroupName = ParamGroupTradePlanner, Order = 13)]
        public bool TradePlannerLineDots { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Label location", Description = "Location of price labels for stop and target lines.",
            GroupName = ParamGroupTradePlanner, Order = 15)]
        public AB_PowerX.TradePlannerLabelLocations TradePlannerLabelLocation { get; set; }

        [NinjaScriptProperty]
        [TypeConverter(typeof(AB_PowerX.EnumDescriptionTypeConverter))]
        [PropertyEditor("NinjaTrader.Gui.Tools.StringStandardValuesEditorKey")]
        [Display(Name = "Label content", Description = "Information displayed by level labels.",
            GroupName = ParamGroupTradePlanner, Order = 16)]
        public AB_PowerX.TradePlannerLabelOptions TradePlannerLabelOption { get; set; }

        [NinjaScriptProperty]
        [Display(Name = "Cumulative risk",
            Description = "Display combined risk of all contacts vs single-contract risk.",
            GroupName = ParamGroupTradePlanner, Order = 17)]
        public bool TradePlannerLabelCumulativeRisk { get; set; }

        // ReSharper restore MemberCanBePrivate.Global

        #endregion

        #region Functions

        /// <summary>
        /// Calculate the Average Daily (Bar) Range. Used for determining stops and targets.
        /// </summary>
        /// <param name="period">Number of days/bars for which an average range should be calculated.</param>
        /// <returns>double: x-period average of the daily/bar range.</returns>
        private double AverageDailyRange(int period)
        {
            var aggregateRange = 0d;
            for (var i = period - 1; i >= 0; i--)
            {
                var range = Bars.GetHigh(CurrentBar - i) - Bars.GetLow(CurrentBar - i);
                aggregateRange += range;
            }

            return aggregateRange / period;
        }

        /// <summary>
        /// Paint a candle body or leave it transparent based on its close vs. open.
        /// </summary>
        /// <param name="barsAgo">Number of bars ago (relative to current bar) to evaluate.</param>
        /// <param name="fill">Brush to use when filling candle body.</param>
        private void ColourCandle(int barsAgo, Brush fill)
        {
            switch (CandleFill)
            {
                case AB_PowerX.CandleFillStyle.FillDown:
                    BarBrush = Close[barsAgo] < Open[barsAgo] ? fill : Brushes.Transparent;
                    break;
                case AB_PowerX.CandleFillStyle.FillUp:
                    BarBrush = Close[barsAgo] > Open[barsAgo] ? fill : Brushes.Transparent;
                    break;
                default:
                    BarBrush = Brushes.Transparent;
                    break;
            }
        }

        /// <summary>
        /// Draw entry markers on current bar based on directionality of the provided PowerXBar.
        /// </summary>
        /// <param name="refBar">PowerXBar upon which trend direction is based.</param>
        private void DrawEntryMarkers(AB_PowerX.PowerXBar refBar)
        {
            if (!TradeShowMarkers || !DisplayTradeHelper) return;

            // do not process if trigger line does not exist (doesn't make sense)
            if (DrawObjects["trigger(" + refBar.Index + ")"] == null) return;

            // ReSharper disable once SwitchStatementHandlesSomeKnownEnumValuesWithDefault
            switch (refBar.Directionality)
            {
                case AB_PowerX.BarTrend.Bullish:
                    Draw.ArrowUp(this, "entry(" + refBar.Index + ")", false, 0,
                        Low[0] - 1 * TickSize, ColourUp);
                    break;
                case AB_PowerX.BarTrend.Bearish:
                    Draw.ArrowDown(this, "entry(" + refBar.Index + ")", false, 0,
                        High[0] + 1 * TickSize, ColourDown);
                    break;
                default:
                    return;
            }
        }

        /// <summary>
        /// Draw stop (risk) and target (reward) lines to allow for trade planning and management.
        /// </summary>
        /// <param name="refBar">PowerXBar triggered as a signal bar upon which calculations are based.</param>
        private void DrawRiskRewardLines(AB_PowerX.PowerXBar refBar)
        {
            if (!DisplayTradePlanner || !DisplayTradeHelper) return;

            // do not process if trigger line does not exist (doesn't make sense)
            if (DrawObjects["trigger(" + refBar.Index + ")"] == null) return;

            var riskTag = "risk(" + refBar.Index + ")";
            var riskLabelTag1 = "riskL1(" + refBar.Index + ")";
            var riskLabelTag2 = "riskL2(" + refBar.Index + ")";
            var riskLabelText1 = string.Empty;
            var riskLabelText2 = string.Empty;

            // determine trade qty based on account balance
            var contractQty = (int)Math.Floor(AccountPercentRisk / 100 * (accountBalance ?? 0) / refBar.RiskDollars);

            // ReSharper disable once SwitchStatementHandlesSomeKnownEnumValuesWithDefault
            switch (TradePlannerLabelOption)
            {
                case AB_PowerX.TradePlannerLabelOptions.QtyPriceSplit:
                    riskLabelText1 = contractQty.ToString();
                    riskLabelText2 = refBar.StopValue.ToString(CultureInfo.InvariantCulture);
                    break;
                case AB_PowerX.TradePlannerLabelOptions.QtyPrice:
                    riskLabelText1 = "";
                    riskLabelText2 = contractQty + " @ " + refBar.StopValue;
                    break;
                case AB_PowerX.TradePlannerLabelOptions.RiskTicks:
                    riskLabelText1 =
                        (TradePlannerLabelCumulativeRisk ? refBar.RiskTicks * contractQty : refBar.RiskTicks) + "T";
                    riskLabelText2 = contractQty + " @ " + refBar.StopValue;
                    break;
                case AB_PowerX.TradePlannerLabelOptions.RiskDollar:
                    riskLabelText1 = "$" + string.Format("{0:0,0.00}", TradePlannerLabelCumulativeRisk
                        ? refBar.RiskDollars * contractQty
                        : refBar.RiskDollars);
                    riskLabelText2 = contractQty + " @ " + refBar.StopValue;
                    break;
                case AB_PowerX.TradePlannerLabelOptions.All:
                    riskLabelText1 =
                        (TradePlannerLabelCumulativeRisk ? refBar.RiskTicks * contractQty : refBar.RiskTicks) + "T|$" +
                        string.Format("{0:0,0.00}", TradePlannerLabelCumulativeRisk
                            ? refBar.RiskDollars * contractQty
                            : refBar.RiskDollars);
                    riskLabelText2 = contractQty + " @ " + refBar.StopValue;
                    break;
            }

            var targetTag = "tgt(" + refBar.Index + ")";
            var targetLabelTag = "tgtLabel(" + refBar.Index + ")";
            var targetLabelText = contractQty + " @ " + refBar.TargetValue.ToString(CultureInfo.InvariantCulture);

            var startX = CurrentBar - refBar.Index;
            var endX = TradePlannerLineLength * -1;

            var labelPosition1 = TradePlannerLabelLocation == AB_PowerX.TradePlannerLabelLocations.Default
                ? endX
                : startX;
            var labelAlignment1 = TradePlannerLabelLocation == AB_PowerX.TradePlannerLabelLocations.Default
                ? TextAlignment.Left
                : TextAlignment.Right;
            var labelPosition2 = TradePlannerLabelLocation == AB_PowerX.TradePlannerLabelLocations.Default
                ? startX
                : endX;
            var labelAlignment2 = TradePlannerLabelLocation == AB_PowerX.TradePlannerLabelLocations.Default
                ? TextAlignment.Right
                : TextAlignment.Left;

            // risk line and label
            Draw.Line(this, riskTag, true, startX, refBar.StopValue, endX, refBar.StopValue, TradePlannerStopLine,
                TradePlannerLineDots ? DashStyleHelper.Dot : DashStyleHelper.Solid, TradePlannerLineThickness);
            if (TradePlannerLabelLocation != AB_PowerX.TradePlannerLabelLocations.None)
            {
                Draw.Text(this, riskLabelTag1, true,
                    riskLabelText1, labelPosition1, refBar.StopValue, 2, Brushes.WhiteSmoke,
                    ChartControl.Properties.LabelFont, labelAlignment1, Brushes.Transparent, TradePlannerStopLine, 70);
                Draw.Text(this, riskLabelTag2, true, riskLabelText2, labelPosition2, refBar.StopValue, 2,
                    Brushes.WhiteSmoke, ChartControl.Properties.LabelFont, labelAlignment2, Brushes.Transparent,
                    TradePlannerStopLine, 70);
            }

            // reward line
            Draw.Line(this, targetTag, true, startX, refBar.TargetValue, endX, refBar.TargetValue,
                TradePlannerTargetLine, TradePlannerLineDots ? DashStyleHelper.Dot : DashStyleHelper.Solid,
                TradePlannerLineThickness);
            if (TradePlannerLabelLocation != AB_PowerX.TradePlannerLabelLocations.None)
                Draw.Text(this, targetLabelTag, true, targetLabelText, labelPosition2, refBar.TargetValue, 2,
                    Brushes.WhiteSmoke,
                    ChartControl.Properties.LabelFont, labelAlignment2, Brushes.Transparent, TradePlannerTargetLine,
                    55);
        }

        /// <summary>
        /// Draw trigger lines based on the provided bar.
        /// </summary>
        /// <param name="refBar">Bar upon which trigger lines are based.</param>
        private void DrawTriggerLines(AB_PowerX.PowerXBar refBar)
        {
            if (!DisplayTradeHelper) return;

            // ReSharper disable once SwitchStatementHandlesSomeKnownEnumValuesWithDefault
            switch (refBar.Directionality)
            {
                case AB_PowerX.BarTrend.Bullish:
                    Draw.Line(this, "trigger(" + refBar.Index + ")", true,
                        CurrentBar - refBar.Index, refBar.ActivationValue,
                        CurrentBar - refBar.Expires, refBar.ActivationValue,
                        TradeBullishTriggerLine,
                        TradeTriggerLineDashed ? DashStyleHelper.Dash : DashStyleHelper.Solid,
                        TradeTriggerLineWidth);
                    break;
                case AB_PowerX.BarTrend.Bearish:
                    Draw.Line(this, "trigger(" + refBar.Index + ")", true,
                        CurrentBar - refBar.Index, refBar.ActivationValue,
                        CurrentBar - refBar.Expires, refBar.ActivationValue,
                        TradeBearishTriggerLine,
                        TradeTriggerLineDashed ? DashStyleHelper.Dash : DashStyleHelper.Solid,
                        TradeTriggerLineWidth);
                    break;
                default:
                    return;
            }
        }

        #endregion


        protected override void OnStateChange()
        {
            // ReSharper disable once SwitchStatementMissingSomeEnumCasesNoDefault
            switch (State)
            {
                case State.SetDefaults:
                    Description = MyIndicatorDescription;
                    Name = string.Format("{0}({1})", MyIndicatorName, MyIndicatorVersion);
                    Calculate = Calculate.OnBarClose;
                    IsOverlay = true;
                    DisplayInDataBox = true;
                    DrawOnPricePanel = true;
                    PaintPriceMarkers = true;
                    ScaleJustification = ScaleJustification.Right;
                    IsSuspendedWhileInactive = true;
                    BarsRequiredToPlot = 0;

                    CustomizeIndicatorValues = false;

                    // default bar colours
                    ColourNeutral = Brushes.DimGray;
                    ColourUp = Brushes.LightGreen;
                    ColourDown = Brushes.LightCoral;

                    // indicator defaults
                    MACDFast = 12;
                    MACDSlow = 26;
                    MACDSignal = 9;
                    StochasticK = 14;
                    StochasticD = 3;
                    StochasticSmooth = 3;
                    RSIPeriod = 7;
                    RSISmooth = 1;

                    // trade helper defaults
                    DisplayTradeHelper = true;

                    // account defaults
                    AccountName = "Sim101";
                    AccountPercentRisk = 2.5f;

                    // trigger line defaults
                    TriggerDisplayOptions = AB_PowerX.TriggerDisplayOptions.RemoveExpired;
                    TradeTriggerDistance = 1;
                    TradeTriggerValidityPeriod = 5;
                    TradeBullishTriggerLine = Brushes.LawnGreen;
                    TradeBearishTriggerLine = Brushes.Firebrick;
                    TradeTriggerLineWidth = 3;
                    TradeTriggerLineDashed = true;
                    TradeShowMarkers = true;

                    // trade planner defaults
                    DisplayTradePlanner = true;
                    TradePlannerLabelOption = AB_PowerX.TradePlannerLabelOptions.RiskTicks;
                    TradePlannerAdrPeriod = 7;
                    TradePlannerTargetMultiple = 3d;
                    TradePlannerStopMultiple = 1.5d;
                    TradePlannerLineLength = TradeTriggerValidityPeriod;
                    TradePlannerStopLine = Brushes.Firebrick;
                    TradePlannerTargetLine = Brushes.LawnGreen;
                    TradePlannerLineThickness = 3;
                    TradePlannerLineDots = true;
                    TradePlannerLabelLocation = AB_PowerX.TradePlannerLabelLocations.Default;
                    TradePlannerLabelCumulativeRisk = false;
                    break;
                case State.Configure:
                    ClearOutputWindow();
                    minBarsNeeded = new[] { MACDFast, MACDSlow, StochasticK, RSIPeriod }.Max();
                    break;
                case State.DataLoaded:
                    account = Account.All.FirstOrDefault(a => a.Name == AccountName);
                    break;
            }
        }

        protected override void OnBarUpdate()
        {
            // display message and return on incompatible chart types
            if (
                ChartBars.Properties.ChartStyleType == ChartStyleType.HollowCandleStick
                || ChartBars.Properties.ChartStyleType == ChartStyleType.KagiLine
                || ChartBars.Properties.ChartStyleType == ChartStyleType.LineOnClose
                || ChartBars.Properties.ChartStyleType == ChartStyleType.Mountain
                || ChartBars.Properties.ChartStyleType == ChartStyleType.PointAndFigure)
            {
                Draw.TextFixed(this, "ErrorText", IncompatibleChartType, TextPosition.BottomLeft,
                    ChartControl.Properties.ChartText, ChartControl.Properties.LabelFont, Brushes.Red,
                    Brushes.Transparent, 0);
                return;
            }

            // return until sufficient bars loaded for indicators
            if (ChartBars == null || ChartBars.Bars == null || Count < minBarsNeeded || CurrentBar < minBarsNeeded)
                return;

            // update account balance
            accountBalance = account.Get(AccountItem.CashValue, Currency.UsDollar);

            // update list of PowerXBars
            powerXBars.Add(new AB_PowerX.PowerXBar(Bars, TickSize, TradeTriggerDistance, CurrentBar,
                TradeTriggerValidityPeriod, MACD(MACDFast, MACDSlow, MACDSignal),
                Stochastics(StochasticD, StochasticK, StochasticSmooth), RSI(RSIPeriod, RSISmooth)));

            // set latest PowerXBar as current one
            var currentPowerXBar = powerXBars.Last();

            // paint bar
            // ReSharper disable SwitchStatementHandlesSomeKnownEnumValuesWithDefault
            switch (ChartBars.Properties.ChartStyleType)
            {
                case ChartStyleType.OHLC:
                    if (currentPowerXBar.Directionality == AB_PowerX.BarTrend.Neutral)
                        BarBrush = ColourNeutral;
                    else
                        BarBrush = currentPowerXBar.Directionality == AB_PowerX.BarTrend.Bullish
                            ? ColourUp
                            : ColourDown;
                    break;
                case ChartStyleType.Box:
                case ChartStyleType.CandleStick:
                case ChartStyleType.Equivolume:
                case ChartStyleType.OpenClose:
                    if (currentPowerXBar.Directionality == AB_PowerX.BarTrend.Neutral)
                    {
                        CandleOutlineBrush = ColourNeutral;
                        ColourCandle(0, ColourNeutral);
                    }
                    else
                    {
                        CandleOutlineBrush = currentPowerXBar.Directionality == AB_PowerX.BarTrend.Bullish
                            ? ColourUp
                            : ColourDown;
                        ColourCandle(0,
                            currentPowerXBar.Directionality == AB_PowerX.BarTrend.Bullish ? ColourUp : ColourDown);
                    }

                    break;
                default:
                    BarBrush = null;
                    CandleOutlineBrush = null;
                    break;
            }

            // ReSharper restore SwitchStatementHandlesSomeKnownEnumValuesWithDefault

            // if not using Trade Helper or this is our first PowerXBar object then we have nothing else to do
            if (!DisplayTradeHelper || powerXBars.Count == 1) return;

            // check if currentBar triggers currentPowerXBar
            var previousPowerXBar = powerXBars[powerXBars.Count - 2];
            if (previousPowerXBar.Directionality == AB_PowerX.BarTrend.Neutral &&
                currentPowerXBar.Directionality != AB_PowerX.BarTrend.Neutral)
            {
                if (TriggerDisplayOptions == AB_PowerX.TriggerDisplayOptions.MostRecent)
                    RemoveDrawObjects();

                // new triggered bar: add to list and draw trigger line
                triggeredBars.Add(currentPowerXBar);
                DrawTriggerLines(currentPowerXBar);

                // set PowerXBar as 'triggered' and draw risk and reward lines
                currentPowerXBar.Trigger(AverageDailyRange(TradePlannerAdrPeriod), TradePlannerStopMultiple,
                    TradePlannerTargetMultiple);
                DrawRiskRewardLines(currentPowerXBar);
            }

            // if no triggered bars, there is nothing to do
            if (triggeredBars.Count == 0) return;

            // set latest TriggeredBars as current one
            var currentTriggeredBar = triggeredBars.Last();

            // check if CurrentBar exceeds currentTriggeredBar --> Activate currentTriggeredBar
            if (CurrentBar <= currentTriggeredBar.Expires && !currentTriggeredBar.Active)
            {
                if (currentTriggeredBar.Directionality == AB_PowerX.BarTrend.Bullish &&
                    High[0] > currentTriggeredBar.ActivationValue)
                {
                    currentTriggeredBar.Activate();
                    DrawEntryMarkers(currentTriggeredBar);
                }
                else if (currentTriggeredBar.Directionality == AB_PowerX.BarTrend.Bearish &&
                         Low[0] < currentTriggeredBar.ActivationValue)
                {
                    currentTriggeredBar.Activate();
                    DrawEntryMarkers(currentTriggeredBar);
                }
            }

            // neutralization: if currentPowerXBar ( ~ CurrentBar) is neutral, then the immediately preceding TriggeredBar is invalidated IFF not already Active
            if (currentPowerXBar.Directionality == AB_PowerX.BarTrend.Neutral && !currentTriggeredBar.Active)
            {
                RemovePowerXBar(currentTriggeredBar, triggeredBars);
            }

            // current trigger bar invalidates previous one IFF not Active
            if (triggeredBars.Count >= 2)
            {
                var previousTriggeredBar = triggeredBars[triggeredBars.Count - 2];
                if (!previousTriggeredBar.Active)
                    RemovePowerXBar(previousTriggeredBar, triggeredBars);
            }

            // clean-up expired triggered bars
            var expiredTriggeredBars = triggeredBars.FindAll(x =>
                x.Index < currentTriggeredBar.Index && CurrentBar > x.Expires && x.Active == false);
            foreach (var expiredTriggeredBar in expiredTriggeredBars)
            {
                if (TriggerDisplayOptions == AB_PowerX.TriggerDisplayOptions.RemoveExpired)
                    RemovePowerXBar(expiredTriggeredBar, triggeredBars);
                else
                    RemovePowerXBar(expiredTriggeredBar, triggeredBars, false);
            }
        }

        /// <summary>
        /// Remove PowerXBar objects from a specified list and, optionally, remove associated drawing objects.
        /// </summary>
        /// <param name="bar">PowerXBar to remove.</param>
        /// <param name="list">List from which to remove PowerXBar.</param>
        /// <param name="removeDrawing">Remove associated drawing object(s) based on bar index being contained within drawing tag.</param>
        private void RemovePowerXBar(AB_PowerX.PowerXBar bar, ICollection<AB_PowerX.PowerXBar> list,
            bool removeDrawing = true)
        {
            // remove from list
            list.Remove(bar);

            if (!removeDrawing) return;

            var objectsToRemove = DrawObjects.Where(x => x.Tag.Contains(bar.Index.ToString()));
            foreach (var obj in objectsToRemove)
            {
                RemoveDrawObject(obj.Tag);
            }
        }
    }

    /// <summary>
    /// TypeConverter to allow NinjaTrader to dynamically show and hide properties in the property grid based on other boolean property 'toggles'.
    /// </summary>
    public class ShowHideParamsTypeConverter : IndicatorBaseConverter
    {
        private PropertyDescriptorCollection propertyDescriptorCollection;
        private ABPowerX indicator;

        public override PropertyDescriptorCollection GetProperties(ITypeDescriptorContext context, object component,
            Attribute[] attributes)
        {
            indicator = component as ABPowerX;
            propertyDescriptorCollection = base.GetPropertiesSupported(context)
                ? base.GetProperties(context, component, attributes)
                : TypeDescriptor.GetProperties(component, attributes);

            if (indicator == null || propertyDescriptorCollection == null)
                return propertyDescriptorCollection;

            // indicator-related properties
            // ReSharper disable InconsistentNaming
            var MACDFast = propertyDescriptorCollection["MACDFast"];
            var MACDSlow = propertyDescriptorCollection["MACDSlow"];
            var MACDSignal = propertyDescriptorCollection["MACDSignal"];
            var StochasticK = propertyDescriptorCollection["StochasticK"];
            var StochasticD = propertyDescriptorCollection["StochasticD"];
            var StochasticSmooth = propertyDescriptorCollection["StochasticSmooth"];
            var RSIPeriod = propertyDescriptorCollection["RSIPeriod"];
            var RSISmooth = propertyDescriptorCollection["RSISmooth"];
            // ReSharper restore InconsistentNaming

            // trade-helper properties
            var triggerDisplayOptions = propertyDescriptorCollection["TriggerDisplayOptions"];
            var tradeTriggerDistance = propertyDescriptorCollection["TradeTriggerDistance"];
            var tradeTriggerValidity = propertyDescriptorCollection["TradeTriggerValidityPeriod"];
            var tradeBullishTriggerLine = propertyDescriptorCollection["TradeBullishTriggerLine"];
            var tradeBearishTriggerLine = propertyDescriptorCollection["TradeBearishTriggerLine"];
            var tradeTriggerLineWidth = propertyDescriptorCollection["TradeTriggerLineWidth"];
            var tradeTriggerLineDotted = propertyDescriptorCollection["TradeTriggerLineDashed"];
            var tradeShowMarkers = propertyDescriptorCollection["TradeShowMarkers"];

            // trade-planner properties
            var displayTradePlanner = propertyDescriptorCollection["DisplayTradePlanner"];
            var tradePlannerAdrPeriod = propertyDescriptorCollection["TradePlannerAdrPeriod"];
            var tradePlannerStopMultiple = propertyDescriptorCollection["TradePlannerStopMultiple"];
            var tradePlannerTargetMultiple = propertyDescriptorCollection["TradePlannerTargetMultiple"];
            var tradePlannerLineLength = propertyDescriptorCollection["TradePlannerLineLength"];
            var tradePlannerStopLine = propertyDescriptorCollection["TradePlannerStopLine"];
            var tradePlannerTargetLine = propertyDescriptorCollection["TradePlannerTargetLine"];
            var tradePlannerLineThickness = propertyDescriptorCollection["TradePlannerLineThickness"];
            var tradePlannerLineDots = propertyDescriptorCollection["TradePlannerLineDots"];
            var tradePlannerLabelLocation = propertyDescriptorCollection["TradePlannerLabelLocation"];
            var tradePlannerLabelOption = propertyDescriptorCollection["TradePlannerLabelOption"];
            var tradePlannerLabelCumulativeRisk = propertyDescriptorCollection["TradePlannerLabelCumulativeRisk"];

            // properties to hide by default
            propertyDescriptorCollection.Remove(MACDFast);
            propertyDescriptorCollection.Remove(MACDSlow);
            propertyDescriptorCollection.Remove(MACDSignal);
            propertyDescriptorCollection.Remove(StochasticK);
            propertyDescriptorCollection.Remove(StochasticD);
            propertyDescriptorCollection.Remove(StochasticSmooth);
            propertyDescriptorCollection.Remove(RSIPeriod);
            propertyDescriptorCollection.Remove(RSISmooth);

            // toggle: ADD indicator-related properties when below property is TRUE
            if (indicator.CustomizeIndicatorValues)
            {
                propertyDescriptorCollection.Add(MACDFast);
                propertyDescriptorCollection.Add(MACDSlow);
                propertyDescriptorCollection.Add(MACDSignal);
                propertyDescriptorCollection.Add(StochasticK);
                propertyDescriptorCollection.Add(StochasticD);
                propertyDescriptorCollection.Add(StochasticSmooth);
                propertyDescriptorCollection.Add(RSIPeriod);
                propertyDescriptorCollection.Add(RSISmooth);
            }

            // toggle: REMOVE trade-helper related properties when below property is FALSE
            if (!indicator.DisplayTradeHelper)
            {
                propertyDescriptorCollection.Remove(triggerDisplayOptions);
                propertyDescriptorCollection.Remove(tradeTriggerDistance);
                propertyDescriptorCollection.Remove(tradeTriggerValidity);
                propertyDescriptorCollection.Remove(tradeBullishTriggerLine);
                propertyDescriptorCollection.Remove(tradeBearishTriggerLine);
                propertyDescriptorCollection.Remove(tradeTriggerLineWidth);
                propertyDescriptorCollection.Remove(tradeTriggerLineDotted);
                propertyDescriptorCollection.Remove(tradeShowMarkers);

                propertyDescriptorCollection.Remove(displayTradePlanner);
                propertyDescriptorCollection.Remove(tradePlannerAdrPeriod);
                propertyDescriptorCollection.Remove(tradePlannerStopMultiple);
                propertyDescriptorCollection.Remove(tradePlannerTargetMultiple);
                propertyDescriptorCollection.Remove(tradePlannerLineLength);
                propertyDescriptorCollection.Remove(tradePlannerStopLine);
                propertyDescriptorCollection.Remove(tradePlannerTargetLine);
                propertyDescriptorCollection.Remove(tradePlannerLineThickness);
                propertyDescriptorCollection.Remove(tradePlannerLineDots);
                propertyDescriptorCollection.Remove(tradePlannerLabelLocation);
                propertyDescriptorCollection.Remove(tradePlannerLabelOption);
                propertyDescriptorCollection.Remove(tradePlannerLabelCumulativeRisk);
            }

            // toggle: REMOVE trade-planner related properties when below property is FALSE
            // ReSharper disable once InvertIf
            if (!indicator.DisplayTradePlanner)
            {
                propertyDescriptorCollection.Remove(tradePlannerAdrPeriod);
                propertyDescriptorCollection.Remove(tradePlannerStopMultiple);
                propertyDescriptorCollection.Remove(tradePlannerTargetMultiple);
                propertyDescriptorCollection.Remove(tradePlannerLineLength);
                propertyDescriptorCollection.Remove(tradePlannerStopLine);
                propertyDescriptorCollection.Remove(tradePlannerTargetLine);
                propertyDescriptorCollection.Remove(tradePlannerLineThickness);
                propertyDescriptorCollection.Remove(tradePlannerLineDots);
                propertyDescriptorCollection.Remove(tradePlannerLabelLocation);
                propertyDescriptorCollection.Remove(tradePlannerLabelOption);
                propertyDescriptorCollection.Remove(tradePlannerLabelCumulativeRisk);
            }

            return propertyDescriptorCollection;
        }
    }
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private ABPowerX.ABPowerX[] cacheABPowerX;
		public ABPowerX.ABPowerX ABPowerX(AB_PowerX.CandleFillStyle candleFill, bool customizeIndicatorValues, int mACDFast, int mACDSlow, int mACDSignal, int stochasticK, int stochasticD, int stochasticSmooth, int rSIPeriod, int rSISmooth, bool displayTradeHelper, string accountName, float accountPercentRisk, AB_PowerX.TriggerDisplayOptions triggerDisplayOptions, int tradeTriggerDistance, int tradeTriggerValidityPeriod, int tradeTriggerLineWidth, bool tradeTriggerLineDashed, bool tradeShowMarkers, bool displayTradePlanner, int tradePlannerAdrPeriod, double tradePlannerStopMultiple, double tradePlannerTargetMultiple, int tradePlannerLineLength, int tradePlannerLineThickness, bool tradePlannerLineDots, AB_PowerX.TradePlannerLabelLocations tradePlannerLabelLocation, AB_PowerX.TradePlannerLabelOptions tradePlannerLabelOption, bool tradePlannerLabelCumulativeRisk)
		{
			return ABPowerX(Input, candleFill, customizeIndicatorValues, mACDFast, mACDSlow, mACDSignal, stochasticK, stochasticD, stochasticSmooth, rSIPeriod, rSISmooth, displayTradeHelper, accountName, accountPercentRisk, triggerDisplayOptions, tradeTriggerDistance, tradeTriggerValidityPeriod, tradeTriggerLineWidth, tradeTriggerLineDashed, tradeShowMarkers, displayTradePlanner, tradePlannerAdrPeriod, tradePlannerStopMultiple, tradePlannerTargetMultiple, tradePlannerLineLength, tradePlannerLineThickness, tradePlannerLineDots, tradePlannerLabelLocation, tradePlannerLabelOption, tradePlannerLabelCumulativeRisk);
		}

		public ABPowerX.ABPowerX ABPowerX(ISeries<double> input, AB_PowerX.CandleFillStyle candleFill, bool customizeIndicatorValues, int mACDFast, int mACDSlow, int mACDSignal, int stochasticK, int stochasticD, int stochasticSmooth, int rSIPeriod, int rSISmooth, bool displayTradeHelper, string accountName, float accountPercentRisk, AB_PowerX.TriggerDisplayOptions triggerDisplayOptions, int tradeTriggerDistance, int tradeTriggerValidityPeriod, int tradeTriggerLineWidth, bool tradeTriggerLineDashed, bool tradeShowMarkers, bool displayTradePlanner, int tradePlannerAdrPeriod, double tradePlannerStopMultiple, double tradePlannerTargetMultiple, int tradePlannerLineLength, int tradePlannerLineThickness, bool tradePlannerLineDots, AB_PowerX.TradePlannerLabelLocations tradePlannerLabelLocation, AB_PowerX.TradePlannerLabelOptions tradePlannerLabelOption, bool tradePlannerLabelCumulativeRisk)
		{
			if (cacheABPowerX != null)
				for (int idx = 0; idx < cacheABPowerX.Length; idx++)
					if (cacheABPowerX[idx] != null && cacheABPowerX[idx].CandleFill == candleFill && cacheABPowerX[idx].CustomizeIndicatorValues == customizeIndicatorValues && cacheABPowerX[idx].MACDFast == mACDFast && cacheABPowerX[idx].MACDSlow == mACDSlow && cacheABPowerX[idx].MACDSignal == mACDSignal && cacheABPowerX[idx].StochasticK == stochasticK && cacheABPowerX[idx].StochasticD == stochasticD && cacheABPowerX[idx].StochasticSmooth == stochasticSmooth && cacheABPowerX[idx].RSIPeriod == rSIPeriod && cacheABPowerX[idx].RSISmooth == rSISmooth && cacheABPowerX[idx].DisplayTradeHelper == displayTradeHelper && cacheABPowerX[idx].AccountName == accountName && cacheABPowerX[idx].AccountPercentRisk == accountPercentRisk && cacheABPowerX[idx].TriggerDisplayOptions == triggerDisplayOptions && cacheABPowerX[idx].TradeTriggerDistance == tradeTriggerDistance && cacheABPowerX[idx].TradeTriggerValidityPeriod == tradeTriggerValidityPeriod && cacheABPowerX[idx].TradeTriggerLineWidth == tradeTriggerLineWidth && cacheABPowerX[idx].TradeTriggerLineDashed == tradeTriggerLineDashed && cacheABPowerX[idx].TradeShowMarkers == tradeShowMarkers && cacheABPowerX[idx].DisplayTradePlanner == displayTradePlanner && cacheABPowerX[idx].TradePlannerAdrPeriod == tradePlannerAdrPeriod && cacheABPowerX[idx].TradePlannerStopMultiple == tradePlannerStopMultiple && cacheABPowerX[idx].TradePlannerTargetMultiple == tradePlannerTargetMultiple && cacheABPowerX[idx].TradePlannerLineLength == tradePlannerLineLength && cacheABPowerX[idx].TradePlannerLineThickness == tradePlannerLineThickness && cacheABPowerX[idx].TradePlannerLineDots == tradePlannerLineDots && cacheABPowerX[idx].TradePlannerLabelLocation == tradePlannerLabelLocation && cacheABPowerX[idx].TradePlannerLabelOption == tradePlannerLabelOption && cacheABPowerX[idx].TradePlannerLabelCumulativeRisk == tradePlannerLabelCumulativeRisk && cacheABPowerX[idx].EqualsInput(input))
						return cacheABPowerX[idx];
			return CacheIndicator<ABPowerX.ABPowerX>(new ABPowerX.ABPowerX(){ CandleFill = candleFill, CustomizeIndicatorValues = customizeIndicatorValues, MACDFast = mACDFast, MACDSlow = mACDSlow, MACDSignal = mACDSignal, StochasticK = stochasticK, StochasticD = stochasticD, StochasticSmooth = stochasticSmooth, RSIPeriod = rSIPeriod, RSISmooth = rSISmooth, DisplayTradeHelper = displayTradeHelper, AccountName = accountName, AccountPercentRisk = accountPercentRisk, TriggerDisplayOptions = triggerDisplayOptions, TradeTriggerDistance = tradeTriggerDistance, TradeTriggerValidityPeriod = tradeTriggerValidityPeriod, TradeTriggerLineWidth = tradeTriggerLineWidth, TradeTriggerLineDashed = tradeTriggerLineDashed, TradeShowMarkers = tradeShowMarkers, DisplayTradePlanner = displayTradePlanner, TradePlannerAdrPeriod = tradePlannerAdrPeriod, TradePlannerStopMultiple = tradePlannerStopMultiple, TradePlannerTargetMultiple = tradePlannerTargetMultiple, TradePlannerLineLength = tradePlannerLineLength, TradePlannerLineThickness = tradePlannerLineThickness, TradePlannerLineDots = tradePlannerLineDots, TradePlannerLabelLocation = tradePlannerLabelLocation, TradePlannerLabelOption = tradePlannerLabelOption, TradePlannerLabelCumulativeRisk = tradePlannerLabelCumulativeRisk }, input, ref cacheABPowerX);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.ABPowerX.ABPowerX ABPowerX(AB_PowerX.CandleFillStyle candleFill, bool customizeIndicatorValues, int mACDFast, int mACDSlow, int mACDSignal, int stochasticK, int stochasticD, int stochasticSmooth, int rSIPeriod, int rSISmooth, bool displayTradeHelper, string accountName, float accountPercentRisk, AB_PowerX.TriggerDisplayOptions triggerDisplayOptions, int tradeTriggerDistance, int tradeTriggerValidityPeriod, int tradeTriggerLineWidth, bool tradeTriggerLineDashed, bool tradeShowMarkers, bool displayTradePlanner, int tradePlannerAdrPeriod, double tradePlannerStopMultiple, double tradePlannerTargetMultiple, int tradePlannerLineLength, int tradePlannerLineThickness, bool tradePlannerLineDots, AB_PowerX.TradePlannerLabelLocations tradePlannerLabelLocation, AB_PowerX.TradePlannerLabelOptions tradePlannerLabelOption, bool tradePlannerLabelCumulativeRisk)
		{
			return indicator.ABPowerX(Input, candleFill, customizeIndicatorValues, mACDFast, mACDSlow, mACDSignal, stochasticK, stochasticD, stochasticSmooth, rSIPeriod, rSISmooth, displayTradeHelper, accountName, accountPercentRisk, triggerDisplayOptions, tradeTriggerDistance, tradeTriggerValidityPeriod, tradeTriggerLineWidth, tradeTriggerLineDashed, tradeShowMarkers, displayTradePlanner, tradePlannerAdrPeriod, tradePlannerStopMultiple, tradePlannerTargetMultiple, tradePlannerLineLength, tradePlannerLineThickness, tradePlannerLineDots, tradePlannerLabelLocation, tradePlannerLabelOption, tradePlannerLabelCumulativeRisk);
		}

		public Indicators.ABPowerX.ABPowerX ABPowerX(ISeries<double> input , AB_PowerX.CandleFillStyle candleFill, bool customizeIndicatorValues, int mACDFast, int mACDSlow, int mACDSignal, int stochasticK, int stochasticD, int stochasticSmooth, int rSIPeriod, int rSISmooth, bool displayTradeHelper, string accountName, float accountPercentRisk, AB_PowerX.TriggerDisplayOptions triggerDisplayOptions, int tradeTriggerDistance, int tradeTriggerValidityPeriod, int tradeTriggerLineWidth, bool tradeTriggerLineDashed, bool tradeShowMarkers, bool displayTradePlanner, int tradePlannerAdrPeriod, double tradePlannerStopMultiple, double tradePlannerTargetMultiple, int tradePlannerLineLength, int tradePlannerLineThickness, bool tradePlannerLineDots, AB_PowerX.TradePlannerLabelLocations tradePlannerLabelLocation, AB_PowerX.TradePlannerLabelOptions tradePlannerLabelOption, bool tradePlannerLabelCumulativeRisk)
		{
			return indicator.ABPowerX(input, candleFill, customizeIndicatorValues, mACDFast, mACDSlow, mACDSignal, stochasticK, stochasticD, stochasticSmooth, rSIPeriod, rSISmooth, displayTradeHelper, accountName, accountPercentRisk, triggerDisplayOptions, tradeTriggerDistance, tradeTriggerValidityPeriod, tradeTriggerLineWidth, tradeTriggerLineDashed, tradeShowMarkers, displayTradePlanner, tradePlannerAdrPeriod, tradePlannerStopMultiple, tradePlannerTargetMultiple, tradePlannerLineLength, tradePlannerLineThickness, tradePlannerLineDots, tradePlannerLabelLocation, tradePlannerLabelOption, tradePlannerLabelCumulativeRisk);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.ABPowerX.ABPowerX ABPowerX(AB_PowerX.CandleFillStyle candleFill, bool customizeIndicatorValues, int mACDFast, int mACDSlow, int mACDSignal, int stochasticK, int stochasticD, int stochasticSmooth, int rSIPeriod, int rSISmooth, bool displayTradeHelper, string accountName, float accountPercentRisk, AB_PowerX.TriggerDisplayOptions triggerDisplayOptions, int tradeTriggerDistance, int tradeTriggerValidityPeriod, int tradeTriggerLineWidth, bool tradeTriggerLineDashed, bool tradeShowMarkers, bool displayTradePlanner, int tradePlannerAdrPeriod, double tradePlannerStopMultiple, double tradePlannerTargetMultiple, int tradePlannerLineLength, int tradePlannerLineThickness, bool tradePlannerLineDots, AB_PowerX.TradePlannerLabelLocations tradePlannerLabelLocation, AB_PowerX.TradePlannerLabelOptions tradePlannerLabelOption, bool tradePlannerLabelCumulativeRisk)
		{
			return indicator.ABPowerX(Input, candleFill, customizeIndicatorValues, mACDFast, mACDSlow, mACDSignal, stochasticK, stochasticD, stochasticSmooth, rSIPeriod, rSISmooth, displayTradeHelper, accountName, accountPercentRisk, triggerDisplayOptions, tradeTriggerDistance, tradeTriggerValidityPeriod, tradeTriggerLineWidth, tradeTriggerLineDashed, tradeShowMarkers, displayTradePlanner, tradePlannerAdrPeriod, tradePlannerStopMultiple, tradePlannerTargetMultiple, tradePlannerLineLength, tradePlannerLineThickness, tradePlannerLineDots, tradePlannerLabelLocation, tradePlannerLabelOption, tradePlannerLabelCumulativeRisk);
		}

		public Indicators.ABPowerX.ABPowerX ABPowerX(ISeries<double> input , AB_PowerX.CandleFillStyle candleFill, bool customizeIndicatorValues, int mACDFast, int mACDSlow, int mACDSignal, int stochasticK, int stochasticD, int stochasticSmooth, int rSIPeriod, int rSISmooth, bool displayTradeHelper, string accountName, float accountPercentRisk, AB_PowerX.TriggerDisplayOptions triggerDisplayOptions, int tradeTriggerDistance, int tradeTriggerValidityPeriod, int tradeTriggerLineWidth, bool tradeTriggerLineDashed, bool tradeShowMarkers, bool displayTradePlanner, int tradePlannerAdrPeriod, double tradePlannerStopMultiple, double tradePlannerTargetMultiple, int tradePlannerLineLength, int tradePlannerLineThickness, bool tradePlannerLineDots, AB_PowerX.TradePlannerLabelLocations tradePlannerLabelLocation, AB_PowerX.TradePlannerLabelOptions tradePlannerLabelOption, bool tradePlannerLabelCumulativeRisk)
		{
			return indicator.ABPowerX(input, candleFill, customizeIndicatorValues, mACDFast, mACDSlow, mACDSignal, stochasticK, stochasticD, stochasticSmooth, rSIPeriod, rSISmooth, displayTradeHelper, accountName, accountPercentRisk, triggerDisplayOptions, tradeTriggerDistance, tradeTriggerValidityPeriod, tradeTriggerLineWidth, tradeTriggerLineDashed, tradeShowMarkers, displayTradePlanner, tradePlannerAdrPeriod, tradePlannerStopMultiple, tradePlannerTargetMultiple, tradePlannerLineLength, tradePlannerLineThickness, tradePlannerLineDots, tradePlannerLabelLocation, tradePlannerLabelOption, tradePlannerLabelCumulativeRisk);
		}
	}
}

#endregion
